<template>
  <div class="attributes" v-for="item in combination" :key="item.id">
    <div class="name">{{ item.name }}: </div>
    <el-radio-group v-model="attributesValue[item.id]" size="large">
      <el-radio-button :label="e" v-for="e in item.list" :key="e" :disabled="!checkInventory(item.id, e)"/>
    </el-radio-group>
  </div>
</template>

<script setup>
import { reactive,onMounted } from "vue";
const combination = reactive([
  { name: '颜色', id:'color', list: ['红色', '蓝色', '绿色'] },
  { name: '尺码', id:'size', list: ['S', 'M', 'L'] },
  { name: '材质', id:'texture', list: ['棉质', '涤纶'] },
]);

let skuList = reactive([])

const attributesValue = reactive({
  color:'',
  size:'',
  texture:'',
})

// 生成变体组合
const cartesianProduct = () => {
  const array = combination.map(item => item.list.map(itemVal => ({ name: item.name,id:item.id, val: itemVal })));
  const data = [];

  const func = (skuarr = [], i) => {
    for (let j = 0; j < array[i].length; j++) {
      if (i < array.length - 1) {
        skuarr[i] = array[i][j] // 将当前的变体选项加入 skuarr 数组中
        func(skuarr, i + 1) // 递归调用下一层
      } else {
        data.push([...skuarr, array[i][j]])
      }
    }
    return data
  }
  
  let newList = func([], 0);

  // 开始组装每一项变体属性
  return newList.map(item => {
    const attributesValue = {};
    
    item.forEach(({ id, val }) => {
      attributesValue[id] = val;
    });

    return {
      attributes: item,
      attributesValue,
      SKU: item.map(i => i.val).join(),
      stock: Math.floor(Math.random() * 6), // 生成0-5的随机整数库存
      price: 100,
    };
  });
};

const checkInventory = (id, val) => {
  // 构造新的 attributesValue 对象
  const newAttributesValue = { ...attributesValue, [id]: val };

  for (const sku of skuList) {
    let match = true;
    for (const [key, value] of Object.entries(newAttributesValue)) {
      if (value && sku.attributesValue[key] !== value) {
        match = false;
        break;
      }
    }
    if (match && sku.stock > 0) {
      return true;
    }
  }
  
  return false;
}

onMounted(()=> {
  const data = cartesianProduct ()
  skuList.push(...data)
  console.log('skuList: ', skuList);
})
</script>

<style lang="scss" scoped>
.attributes {
  display: flex;
  margin-bottom: 20px;
  .name {
    line-height: 40px;
  }
  .el-radio-button {
    margin: 0 10px;
    --el-radio-button-checked-border-color:none;
    :deep(.el-radio-button__inner) {
      border: 1px solid #e4e7ed;
    }
  }
  .el-radio-button.is-disabled :deep(.el-radio-button__inner){
    background-color: #f2f6fc;
  }
}
</style>
